import logging
import time

from db_facade.db_clients import postgresql


class DB():

    _connectors = {"postgresql": postgresql.get_connection}
    _inserters = {"postgresql": postgresql.insert_rows}
    _fetchers = {"postgresql": postgresql.fetch_many}
    _executers = {"postgresql": postgresql.execute_query}
    _connection_status_providers = {"postgresql": postgresql.is_connection_open}

    def __init__(self, url, db="postgresql"):
        self.db = db
        self.url = url
        self.connection = self.connect()

    def connect(self):
        connector = self._connectors[self.db]
        return connector(self.url)

    def insert_many(self, table_name, data):
        try:
            inserter = self._inserters[self.db]
            inserter(self.connection, table_name, data)
        except Exception as e:
            logging.error(
                f'DB - {self.db} - Insert many failed with error:{e}'
            )

    def fetch_many(self, table_name, limit=None):
        try:
            fetcher = self._fetchers[self.db]
            fetcher(self.connection, table_name, limit)
        except Exception as e:
            logging.error(
                f'DB - {self.db} - Fetch many failed with error:{e}'
            )

    def execute_query(self, query, data=None):
        try:
            executer = self._executers[self.db]
            executer(self.connection, query, data)
        except Exception as e:
            logging.error(
                f'DB - {self.db} - Execute query failed with error:{e}'
            )

    def is_connection_open(self):
        connection_status_provider = self._connection_status_providers[self.db]
        return connection_status_provider(self.connection)

    def keep_connection_open(self):

        while True:
            # Wait 5 mins
            time.sleep(60 * 5)

            if not self.is_connection_open():
                try:
                    self.retry_to_connect()
                except:
                    return

    def retry_to_connect(self):
        retry_limit = 10
        for retry_count in range(retry_limit):
            try:
                logging.info(
                    f'DB - {self.db} - Retrying to connect ({retry_count}/{retry_limit})'
                )
                self.connection = self.connect()
                if self.is_connection_open():
                    return

                # Wait 5 mins
                time.sleep(60 * 5)

            except Exception as e:
                logging.error(
                    f'DB - {self.db} - Failed while retrying to connect, with error:{e}'
                )
                raise Exception('Retry Failed!')

        logging.error(f'DB - {self.db} - Reached retry limit')
        raise Exception('Reached retry limit')
